// convenience_ext.cpp
//
#include "convenience_ext.hpp"


namespace boost
{
	namespace filesystem
	{
		// Simple file/pattern matcher which returns true if an input file matches the specified pattern.
		// The pattern can include the wildchars ? and * and other fixed filename parts.
		//
		bool is_match(const path & ph, const char *pattern)
		{
			char					*cp, *mp;
			const char		*szfn;


			if (!exists(ph) || is_directory(ph))
        boost::throw_exception(filesystem_error("boost::filesystem::is_match", ph, "invalid path or path is not a file "));

			std::string&	strph = ph.leaf();
			szfn = strph.c_str();

			while ((*szfn) && (*pattern != '*'))
			{
				if ((BOOST_FILENAME_CHAR(*pattern) != BOOST_FILENAME_CHAR(*szfn)) && (*pattern != '?'))
					return false;

				pattern++;
				szfn++;
			}
				
			while (*szfn)
			{
				if (*pattern == '*')
				{
					if (!*++pattern)
						return true;

					mp = (char *) pattern;
					cp = (char *) szfn+1;
				}
				else
				{
					if ((BOOST_FILENAME_CHAR(*pattern) == BOOST_FILENAME_CHAR(*szfn)) || (*pattern == '?'))
					{
						pattern++;
						szfn++;
					}
					else
					{
						pattern = mp;
						szfn = cp++;
					}
				}
			}
				
			while (*pattern == '*')
				pattern++;


			return *pattern == '\0';
		}


#ifdef BOOST_FILESYSTEM_FILEDATES

		filedates file_dates(const path & ph)
		{
			filedates								sFileDates;
			int											fd ;				// file descriptor
			struct stat							buf;
			struct tm*							ptm;


			if (!exists(ph) || is_directory(ph))
        boost::throw_exception(filesystem_error("boost::filesystem::is_match", ph, "invalid path or path is not a file "));

			if ((fd = open(ph.native_file_string().c_str(), O_RDONLY)) != -1)
			{
				// Fill the stat buffer and close the file
				//
				// Warning: it appears the the date last access is modified at the close (at least in Windows)
				//          so running this routine twice on the same file may yield different last access dates.
				//
				fstat(fd, &buf);
				close(fd);

				// Set the dates
				//
				ptm = localtime(&buf.st_ctime);
				sFileDates.m_file_created = boost::posix_time::ptime(boost::gregorian::date(ptm->tm_year + 1900, ptm->tm_mon + 1, ptm->tm_mday), boost::posix_time::time_duration(ptm->tm_hour, ptm->tm_min, ptm->tm_sec));

				ptm = localtime(&buf.st_atime);
				sFileDates.m_file_last_accessed = boost::posix_time::ptime(boost::gregorian::date(ptm->tm_year + 1900, ptm->tm_mon + 1, ptm->tm_mday), boost::posix_time::time_duration(ptm->tm_hour, ptm->tm_min, ptm->tm_sec));

				ptm = localtime(&buf.st_mtime);
				sFileDates.m_file_last_modified = boost::posix_time::ptime(boost::gregorian::date(ptm->tm_year + 1900, ptm->tm_mon + 1, ptm->tm_mday), boost::posix_time::time_duration(ptm->tm_hour, ptm->tm_min, ptm->tm_sec));
			}


			return sFileDates;
		}

#endif

	}	// end namespace filesystem

}	// end namespace boost